package dk.nsp.idsas.service;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import dk.nsp.idsas.setup.DataSourceTestConfiguration;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.setup.ServiceTestConfiguration;

import static org.junit.Assert.assertEquals;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {
        DataSourceTestConfiguration.class,
        ServiceTestConfiguration.class})
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
public class BlurringLookupServiceTest {

    @Autowired
    BlurringService blurringService;

    @Autowired
    LookupService lookupService;

    private final IdsasActor systembruger = IdsasActor.createSystem("5678", "TestSystem");

    @Test
    public void createOneBlurring() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        LocalDateTime localDateTime = LocalDateTime.now().plusDays(1);

        // Act
        blurringService.createBlurring(systembruger, patientIdType, patientId, toGregorian(localDateTime));

        // Assert
        List<String> orgs = lookupService.getActiveBlurrings(
                systembruger, patientId, patientIdType);
        assertEquals(1, orgs.size());
    }

    @Test
    public void createThreeBlurringsForSamePersonAndSameOrg() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        LocalDateTime localDateTime = LocalDateTime.now().plusDays(1);

        // Act
        blurringService.createBlurring(systembruger, patientIdType, patientId, toGregorian(localDateTime));
        blurringService.createBlurring(systembruger, patientIdType, patientId, toGregorian(localDateTime));
        blurringService.createBlurring(systembruger, patientIdType, patientId, toGregorian(localDateTime));

        // Assert
        List<String> orgs = lookupService.getActiveBlurrings(
                systembruger, patientId, patientIdType);

        // Kun seneste blurring er aktiv
        assertEquals(1, orgs.size());
    }

    @Test
    public void createThreeBlurringsForSamePersonDifferentOrgs() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        LocalDateTime localDateTime = LocalDateTime.now().plusDays(1);

        // Act
        blurringService.createBlurring(IdsasActor.createSundhedsFaglig("5678"),
                patientIdType, patientId, toGregorian(localDateTime));
        blurringService.createBlurring(IdsasActor.createSundhedsFaglig("6666"),
                patientIdType, patientId, toGregorian(localDateTime));
        blurringService.createBlurring(IdsasActor.createSundhedsFaglig("7777"),
                patientIdType, patientId, toGregorian(localDateTime));

        // Assert
        List<String> orgs = lookupService.getActiveBlurrings(
                systembruger, patientId, patientIdType);
        assertEquals(3, orgs.size());
    }

    @Test
    public void createBlurringInTheFarFuture() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        LocalDateTime localDateTime = LocalDateTime.now().plusYears(5);

        // Act
        blurringService.createBlurring(systembruger, patientIdType, patientId, toGregorian(localDateTime));

        // Assert
        List<String> orgs = lookupService.getActiveBlurrings(
                systembruger, patientId, patientIdType);
        assertEquals(1, orgs.size());
    }

    @Test
    public void pastBlurringsAreNotActive() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        String patientIdType = "CPR";
        LocalDateTime localDateTime = LocalDateTime.now().minusYears(5);

        // Act
        blurringService.createBlurring(systembruger, patientIdType, patientId, toGregorian(localDateTime));

        // Assert
        List<String> orgs = lookupService.getActiveBlurrings(
                systembruger, patientId, patientIdType);
        assertEquals(0, orgs.size());
    }

    private GregorianCalendar toGregorian(LocalDateTime datetime) {
        GregorianCalendar gregorianCalendar = (GregorianCalendar) GregorianCalendar.getInstance();
        gregorianCalendar.setTime(Date.from(datetime.toInstant(ZoneOffset.UTC)));
        return gregorianCalendar;
    }

}
